import * as filters from "../filter";
import prisma, { MatchingAlgorithms } from "../../../generated/prisma";

interface FilterTestCase {
    input: string,
    expected: boolean;
}

const templateFilter: prisma.Filter = {
    id: 0,
    guild_id: "",
    name: "",
    value: "",
    matching_algorithm: MatchingAlgorithms.ALL,
    is_insensitive: false,
    is_whitelist: false,
    created_at: new Date(),
    updated_at: new Date()

};

const runFilterTest = (filter: prisma.Filter, testCases: FilterTestCase[]) => {
    for (const { input, expected } of testCases) {
        test(`Input: ${input}`, () => {
            const result = filters.mapAlgorithmToFunction(filter, input);
            expect(result).toBe(expected);
        });
    }
};

describe("Match: ALL (Case Sensitive)", () => {
    const filter: prisma.Filter = {
        ...templateFilter,
        name: "Block All Words - CASE SENSITIVE",
        value: String.raw`one TWO threE`,
        matching_algorithm: MatchingAlgorithms.ALL,
        is_insensitive: false
    };

    const testCases: FilterTestCase[] = [
        {
            input: "one two three",
            expected: false
        },
        {
            input: "one threE four five",
            expected: false
        },
        {
            input: "two four one six three",
            expected: false
        },
        {
            input: "one TWO threE four five",
            expected: true
        },
        {
            input: "nine ten seven eight one",
            expected: false
        },
        {
            input: "five four threE TWO one",
            expected: true
        },
        {
            input: "six nine one three eight",
            expected: false
        }
    ];

    runFilterTest(filter, testCases);
});

describe("Match: ANY", () => { 
    const filter: prisma.Filter = {
        ...templateFilter,
        name: "Block American Politics",
        value: String.raw`trump biden democrat republican gop dnc kamala harris`,
        matching_algorithm: MatchingAlgorithms.ANY,
        is_insensitive: true
    };

    const testCases: FilterTestCase[] = [
        {
            input: "Republicans float new tax breaks for tips, local taxes in Trump budget package",
            expected: true  // Contains 'republican' and 'trump'.
        },
        {
            input: "Biden’s Approval Rating Dips Amid Economic Concerns",
            expected: true  // Contains 'biden'.
        },
        {
            input: "GOP Governors Call for Tougher Border Security Measures",
            expected: true  // Contains 'gop'.
        },
        {
            input: "Biden Announces New Initiative to Tackle Climate Change",
            expected: true  // Contains 'biden'.
        },
        {
            input: "Joe Biden gives thoughts on Donald Trump and Kamala Harris in first interview since leaving office",
            expected: true  // Contains 'biden', 'trump', 'kamala' and 'harris'.
        },
        {
            input: "UK Prime Minister Keir Starmer hails limited US-UK trade deal, but 10% duties remain",
            expected: false
        },
        {
            input: "Scientists discover new species of fish in Pacific Ocean",
            expected: false
        },
        {
            input: "Federal Reserve signals cautious approach to interest rate cuts in 2025",
            expected: false
        }
    ];

    runFilterTest(filter, testCases);
});

describe("Match: LITERAL", () => {  });

describe("Match: REGEX", () => {
    const filter: prisma.Filter = {
        ...templateFilter,
        name: "Block American Politics",
        value: String.raw`\b(trump|biden|democrat|republican|gop|dnc|kamala|harris)\b`,
        matching_algorithm: MatchingAlgorithms.REGEX,
        is_insensitive: true
    };

    const testCases: FilterTestCase[] = [
        {
            input: "Republicans float new tax breaks for tips, local taxes in Trump budget package",
            expected: true  // Contains 'republican' and 'trump'.
        },
        {
            input: "Biden’s Approval Rating Dips Amid Economic Concerns",
            expected: true  // Contains 'biden'.
        },
        {
            input: "GOP Governors Call for Tougher Border Security Measures",
            expected: true  // Contains 'gop'.
        },
        {
            input: "Biden Announces New Initiative to Tackle Climate Change",
            expected: true  // Contains 'biden'.
        },
        {
            input: "Joe Biden gives thoughts on Donald Trump and Kamala Harris in first interview since leaving office",
            expected: true  // Contains 'biden', 'trump', 'kamala' and 'harris'.
        },
        {
            input: "UK Prime Minister Keir Starmer hails limited US-UK trade deal, but 10% duties remain",
            expected: false
        },
        {
            input: "Scientists discover new species of fish in Pacific Ocean",
            expected: false
        },
        {
            input: "Federal Reserve signals cautious approach to interest rate cuts in 2025",
            expected: false
        }
    ];

    runFilterTest(filter, testCases);
});

describe("Match: FUZZY", () => {
    const filter: prisma.Filter = {
        ...templateFilter,
        name: "Hello World Filter",
        value: String.raw`hello world`,
        matching_algorithm: MatchingAlgorithms.FUZZY,
        is_insensitive: true
    };

    const testCases: FilterTestCase[] = [
        {
            input: "",
            expected: false
        },
        {
            input: "hello world",
            expected: true  // exact match
        },
        {
            input: "Hello world",
            expected: true
        },
        {
            input: "hello World",
            expected: true 
        },
        {
            input: "hello",
            expected: true
        },
        {
            input: "world",
            expected: true
        },
        {
            input: "hello there world",
            expected: false
        },
        {
            input: "hello worl",
            expected: true
        },
    ];

    runFilterTest(filter, testCases);
});
