import fuzz from "fuzzball";
import { Filter, MatchingAlgorithms } from "../../generated/prisma";

function splitWords(filterValue: string): string[] {
    const findTerms = [...filterValue.matchAll(/"([^"]+)"|(\S+)/g)];
    return findTerms.map(value => {
        const term = value[1] || value[2];
        return term.trim()
            .replace(/\s+/g, "\\s+")                 // Replace whitespace with equivelant regex characters
            .replace(/[.*+?^${}()|[\]\\]/g, '\\$&')  // Escape common regex characters
    });
}

// Includes all input words (space separated)
export const all = (filter: Filter, input: string) => {
    try {
        const flags = filter.is_insensitive ? "i" : "";
        return splitWords(filter.value)
            .every(word => new RegExp(String.raw`\b(${word})\b`, flags).test(input));
    } catch (error) {
        console.error(`ALL: Invalid regex pattern: ${filter.value}`, error);
        return false;
    }
};

// Includes any input word (space separated)
export const any = (filter: Filter, input: string) => {
    try {
        const flags = filter.is_insensitive ? "i" : "";
        const filterWords = splitWords(filter.value).toString().replace(/,/g, "|");
        const filterValue = String.raw`\b(${filterWords})\b`
        return new RegExp(filterValue, flags).test(input);
    } catch (error) {
        console.error(`ANY: Invalid regex pattern: ${filter.value}`, error);
        return false;
    }
};

// Includes exact input string
export const literal = (filter: Filter, input: string) => {
    try {
        const filterValue = filter.is_insensitive ? filter.value.toLowerCase() : input;
        const inputValue = filter.is_insensitive ? input.toLowerCase() : input;
        return inputValue.includes(filterValue);
    } catch (error) {
        console.error(`LITERAL: Invalid regex pattern: ${filter.value}`, error);
        return false;
    }
};

// Matches given regex pattern
export const regex = (filter: Filter, input: string) => {
    try {
        const flags = filter.is_insensitive ? "i" : "";
        return new RegExp(filter.value, flags).test(input);
    } catch (error) {
        console.error(`REGEX: Invalid regex pattern: ${filter.value}`, error);
        return false;
    }
};

export const fuzzy = (filter: Filter, input: string) => {
    let filterValue = filter.value.replace(/[^\w\s]/g, "");
    let inputValue = input.replace(/[^\w\s]/g, "");

    // NOTE: it seems that case sensitivity is ignored for fuzzy matches... ?
    if (filter.is_insensitive) {
        filterValue = filterValue.toLowerCase();
        inputValue = inputValue.toLowerCase();
    }

    const ratio = fuzz.partial_ratio(filterValue, inputValue);
    return ratio > 90;
};

export const mapAlgorithmToFunction = (filter: Filter, input: string) => {
    switch (filter.matching_algorithm) {
        case MatchingAlgorithms.ALL: return all(filter, input);
        case MatchingAlgorithms.ANY: return any(filter, input);
        case MatchingAlgorithms.EXACT: return literal(filter, input);
        case MatchingAlgorithms.REGEX: return regex(filter, input);
        case MatchingAlgorithms.FUZZY: return fuzzy(filter, input);
        default:
            throw new Error(`Unknown algorithm: ${filter.matching_algorithm}`);
    }
};